/* board.h Copyright 2001 Robert Hilchie */

#include "pattern.h"

#define ROWS 4            // The number of rows in the puzzle
#define COLS 4            // The number of columns in the puzzle
#define TILES (ROWS*COLS) // The number of tiles in the puzzle

// This is a straight-forward representation of the puzzle, used for
// inputting a configuration and generating move tables.
class Board
{
  public:
    Board();

    // Set to the solved configuration
    void home();

    // Input a configuration
    bool input();

    // Determine whether the current configuration can be solved
    bool validate();

    // Perform one of the four possible moves
    bool moveup(bool &count);
    bool movedown(bool &count);
    bool moveleft(bool &count);
    bool moveright(bool &count);

    // Do a moved indexed by the number 'm'
    bool move(unsigned m, bool &count);

    // Do a move and print it out
    void moveandprint(unsigned m);

    // Given a pattern, compute the index of the current configuration
    Index index(Pattern &pattern);

    // Given a pattern and index, create the corresponding configuration
    void unindex(Index idx, Pattern &pattern);

    // Conjugate the current configuration with a horizontal reflection
    void hreflect();

    // Conjugate the current configuration with a vertical reflection
    void vreflect();

#ifdef DEBUG
    void output();
#endif

  private:
    // Swap the tiles t1 and t2
    void swaptiles(Tile t1, Tile t2);

    // Swap the tiles at positions (x1, y1) and (x2, y2)
    void swappos(unsigned y1, unsigned x1, unsigned y2, unsigned x2);

    // tiles[r][c] contains the tile number at row r and column c
    Tile tiles[ROWS][COLS];

    // tilex[t] contains the column number where tile t is located
    unsigned tilex[TILES];

    // tiley[t] contains the row number where tile t is located
    unsigned tiley[TILES];

    // Which tile was moved in the previous move
    unsigned lastmoved;

    // Which column the next move printed will appear in
    unsigned column;
};
