/* pattern.h Copyright 2001 Robert Hilchie */

// A tile's number from 0 to 15, where 0 represents the blank tile.
typedef unsigned char Tile;
#define BLANKTILE     0   // Tile number assigned to the blank
#define DONTCARETILE 99   // A tile not included in a pattern

// In the move table, indicates when a move is not possible
#define NOMOVE  ((unsigned)-1)

// An index into the pattern database or move table, representing the
// configuration of the tiles of the pattern.
typedef unsigned Index;

// Array for each configuration of the pattern, the new configuration's
// index after each possible move.
typedef Index Movetable[4];

// Array for each configuration of the pattern, contains 1 if the given
// move affects a tile in the pattern, 0 otherwise
typedef unsigned char Counttable[4];

// A pattern consisting of a subset of the tiles, and derived information
// such as the pattern database and move table
class Pattern
{
  public:
    // tls is an array of tiles in the pattern, tc is the size of tls
    Pattern(unsigned tc, Tile *tls);

    ~Pattern();

    // Returns number of tiles in the pattern
    inline unsigned size() {return tilecount;};

    // Returns the 'i'th tile in the pattern
    inline Tile itile(unsigned i) {return tiles[i];};

    // Returns a pointer to the move table
    inline Movetable *getmovetable() {return movetable;};

    // Returns a pointer to the pattern database
    inline unsigned char *getdbase() {return dbase;};

    // Create the move table
    bool makemovetable();

    // Create the pattern database
    bool makedbase();

  private:
    // Do a breadth-first search pass on the pattern database
    bool bfspass(unsigned level, unsigned baselevel);

    // Number of tiles in the pattern
    unsigned tilecount;

    // Array of tiles in the pattern
    Tile     *tiles;

    // Number of elements in the pattern database, or rows in the move table
    Index tablesize;

    // The move table (see Movetable above)
    Movetable *movetable;

    // The count table (see Counttable above)
    Counttable *counttable;

    // The pattern database. For each configuration of the tiles of the
    // pattern (including the blank), it contains the minimum number of
    // moves (counting only moves affecting the pattern's tiles excluding
    // the blank) required to move those tiles and the blank into their
    // home positions.
    unsigned char *dbase;

    // Counts number of pattern database entries filled so far
    unsigned dbcount;
};
